@@ -5,7 +5,15 @@ use std::convert::TryFrom;
 use num_enum::TryFromPrimitive;
 use crate::SystemExclusiveData;
 use crate::k5000::morf::Loop;
-use crate::k5000::{RangedValue, RangeKind};
+use crate::k5000::{UnsignedLevel, SignedLevel, UnsignedDepth};
+
+// Semantic types
+pub type EnvelopeRate = UnsignedLevel;
+pub type EnvelopeLevel = SignedLevel;
+pub type EnvelopeDepth = SignedLevel;
+pub type LfoSpeed = UnsignedLevel;
+pub type LfoDepth = UnsignedDepth;
+pub type Bias = SignedLevel;
 
 /// Formant filter envelope mode.
 #[derive(Debug, Eq, PartialEq, Copy, Clone, TryFromPrimitive)]
@@ -21,15 +29,15 @@ impl Default for Mode {
 
 /// Envelope segment.
 pub struct EnvelopeSegment {
-    pub rate: RangedValue,  // 0~127
-    pub level: RangedValue, // -63(1)~+63(127)
+    pub rate: EnvelopeRate,  // 0~127
+    pub level: EnvelopeLevel, // -63(1)~+63(127)
 }
 
 impl Default for EnvelopeSegment {
     fn default() -> Self {
         EnvelopeSegment {
-            rate: RangedValue::from_int(RangeKind::PositiveLevel, 0),
-            level: RangedValue::from_int(RangeKind::SignedLevel, 0),
+            rate: EnvelopeRate::from(0),
+            level: EnvelopeLevel::from(0),
         }
     }
 }
@@ -37,8 +45,8 @@ impl Default for EnvelopeSegment {
 impl SystemExclusiveData for EnvelopeSegment {
     fn from_bytes(data: Vec<u8>) -> Self {
         EnvelopeSegment {
-            rate: RangedValue::from_byte(RangeKind::PositiveLevel, data[0]),
-            level: RangedValue::from_byte(RangeKind::SignedLevel, data[1]),
+            rate: EnvelopeRate::from(data[0]),
+            level: EnvelopeLevel::from(data[1]),
         }
     }
 
@@ -54,8 +62,8 @@ pub struct Envelope {
     pub decay2: EnvelopeSegment,
     pub release: EnvelopeSegment,
     pub decay_loop: Loop,
-    pub velocity_depth: RangedValue,
-    pub ks_depth: RangedValue,
+    pub velocity_depth: EnvelopeDepth,
+    pub ks_depth: EnvelopeDepth,
 }
 
 impl Default for Envelope {
@@ -66,8 +74,8 @@ impl Default for Envelope {
             decay2: Default::default(),
             release: Default::default(),
             decay_loop: Default::default(),
-            velocity_depth: RangedValue::from_int(RangeKind::SignedLevel, 0),
-            ks_depth: RangedValue::from_int(RangeKind::SignedLevel, 0),
+            velocity_depth: EnvelopeDepth::from(0),
+            ks_depth: EnvelopeDepth::from(0),
         }
     }
 }
@@ -80,8 +88,8 @@ impl SystemExclusiveData for Envelope {
             decay2: EnvelopeSegment::from_bytes(data[4..6].to_vec()),
             release: EnvelopeSegment::from_bytes(data[6..8].to_vec()),
             decay_loop: Loop::try_from(data[8]).unwrap(),
-            velocity_depth: RangedValue::from_byte(RangeKind::SignedLevel, data[9]),
-            ks_depth: RangedValue::from_byte(RangeKind::SignedLevel, data[10]),
+            velocity_depth: EnvelopeDepth::from(data[9]),
+            ks_depth: EnvelopeDepth::from(data[10]),
         }
     }
 
@@ -92,7 +100,13 @@ impl SystemExclusiveData for Envelope {
         result.extend(self.decay1.to_bytes());
         result.extend(self.decay2.to_bytes());
         result.extend(self.release.to_bytes());
-        result.extend(vec![self.decay_loop as u8, self.velocity_depth.as_byte(), self.ks_depth.as_byte()]);
+        result.extend(
+            vec![
+                self.decay_loop as u8,
+                self.velocity_depth.as_byte(),
+                self.ks_depth.as_byte()
+            ]
+        );
 
         result
     }
@@ -113,17 +127,17 @@ impl Default for LfoShape {
 
 /// Formant filter LFO.
 pub struct Lfo {
-    pub speed: RangedValue,
+    pub speed: LfoSpeed,
     pub shape: LfoShape,
-    pub depth: RangedValue,
+    pub depth: LfoDepth,
 }
 
 impl Default for Lfo {
     fn default() -> Self {
         Lfo {
-            speed: RangedValue::from_int(RangeKind::PositiveLevel, 0),
+            speed: LfoSpeed::from(0),
             shape: Default::default(),
-            depth: RangedValue::from_int(RangeKind::UnsignedLevel, 0),
+            depth: LfoDepth::from(0),
         }
     }
 }
@@ -131,9 +145,9 @@ impl Default for Lfo {
 impl SystemExclusiveData for Lfo {
     fn from_bytes(data: Vec<u8>) -> Self {
         Lfo {
-            speed: RangedValue::from_byte(RangeKind::PositiveLevel, data[0]),
+            speed: LfoSpeed::from(data[0]),
             shape: LfoShape::try_from(data[1]).unwrap(),
-            depth: RangedValue::from_byte(RangeKind::UnsignedLevel, data[2]),
+            depth: LfoDepth::from(data[2]),
         }
     }
 
@@ -144,9 +158,9 @@ impl SystemExclusiveData for Lfo {
 
 /// Formant filter settings.
 pub struct FormantFilter {
-    pub bias: RangedValue,
+    pub bias: Bias,
     pub mode: Mode,
-    pub envelope_depth: RangedValue,
+    pub envelope_depth: EnvelopeDepth,
     pub envelope: Envelope,
     pub lfo: Lfo,
 }
@@ -154,9 +168,9 @@ pub struct FormantFilter {
 impl Default for FormantFilter {
     fn default() -> Self {
         FormantFilter {
-            bias: RangedValue::from_int(RangeKind::SignedLevel, 0),
+            bias: Bias::from(0),
             mode: Default::default(),
-            envelope_depth: RangedValue::from_int(RangeKind::SignedLevel, 0),
+            envelope_depth: EnvelopeDepth::from(0),
             envelope: Default::default(),
             lfo: Default::default(),
         }
@@ -166,9 +180,9 @@ impl Default for FormantFilter {
 impl SystemExclusiveData for FormantFilter {
     fn from_bytes(data: Vec<u8>) -> Self {
         FormantFilter {
-            bias: RangedValue::from_byte(RangeKind::SignedLevel, data[0]),
+            bias: Bias::from(data[0]),
             mode: Mode::try_from(data[1]).unwrap(),
-            envelope_depth: RangedValue::from_byte(RangeKind::SignedLevel, data[2]),
+            envelope_depth: EnvelopeDepth::from(data[2]),
             envelope: Envelope::from_bytes(data[3..14].to_vec()),
             lfo: Lfo::from_bytes(data[14..].to_vec()),
         }
@@ -177,7 +191,13 @@ impl SystemExclusiveData for FormantFilter {
     fn to_bytes(&self) -> Vec<u8> {
         let mut result: Vec<u8> = Vec::new();
 
-        result.extend(vec![self.bias.as_byte(), self.mode as u8, self.envelope_depth.as_byte()]);
+        result.extend(
+            vec![
+                self.bias.as_byte(),
+                self.mode as u8,
+                self.envelope_depth.as_byte()
+            ]
+        );
         result.extend(self.envelope.to_bytes());
         result.extend(self.lfo.to_bytes());
 
