@@ -26,15 +26,6 @@ impl StringUtils for String {
     }
 }
 
-/// Gets the bit at position `n`. Bits are numbered from 0 (least significant) to 31 (most significant).
-fn get_bit_at(input: u32, n: u8) -> bool {
-    if n < 32 {
-        input & (1 << n) != 0
-    } else {
-        false
-    }
-}
-
 fn every_nth_byte(v: &Vec<u8>, n: usize, start: usize) -> Vec<u8> {
     let mut buf: Vec<u8> = Vec::new();
 
